/**
 * \file bch.h
 *
 * \brief Mini librairy to manipulate binary BCH codes parameters,
 *        for code based steganography.
 *
 * \author Morgan Barbier <morgan.barbier@ensicaen.fr>
 *
 * \version 1.0
 *
 * \date 2014-10-15
 *
 * \copyright This file is part of stegocurve.

 stegocurve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 stegocurve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with stegocurve. If not, see <http://www.gnu.org/licenses/>.

 */

#ifndef __BCH_H__
#define __BCH_H__

#include "codes/code.h"
#include <math.h>

/**
 * \fn void init_BCH2_code(Code *C, const uint parameter);
 *
 * \brief initialize the code C with the parameters of a binary
 *        2-corrector BCH code of parameter 'parameter'. This initialization
 *        permits ONLY to use this code in the worst case.
 *
 * \warning Intialisation for binary 2-corrector BCH code for the worst
 *          case ONLY.
 *
 * \param[out] C the code to initialize
 * \param[in] parameter of the binary 2-corrector BCH code
 */
void init_BCH2_code(Code *C, const uint parameter);
/**
 * \fn void init_BCH2_codes(Code *C, const uint start, const uint nb);
 *
 * \brief initialize the codes array C of size nb with the parameters of binary
 *        2-corrector BCH codes of smallest parameter is 'parameter'. 
 *        This initialization permits ONLY to use these codes in the worst case.
 *
 * \warning Intialisation for binary 2-corrector BCH codes for the worst
 *          case ONLY.
 *
 * \param[out] C the codes array to initialize
 * \param[in] start the smallest parameter of binary 2-corrector BCH codes
 * \param[in] nb the number of codes in the array
 */
void init_BCH2_codes(Code *C, const uint start, const uint nb);
/**
 * \fn void BCH2_codes_WC(const char* filename, const uint start, const uint nb);
 *
 * \brief Create or erase the file filename, and print inside the worst cases
 *        of nb stegoschemes based on binary 2-corrector BCH codes which has the 
 *        smallest parameter start.
 *
 * \param[in] filename the name of the output file
 * \param[in] start the smallest parameter of the binary 2-corrector BCH codes
 * \param[in] nb the number of the binary 2-corrector BCH codes treated
 */
void BCH2_codes_WC(const char* filename, const uint start, const uint nb);


/**
 * \fn void init_PBCH2_code(Code *C, const uint parameter);
 *
 * \brief initialize the code C with the parameters of a punctured binary
 *        2-corrector BCH code of parameter 'parameter'. This initialization
 *        permits ONLY to use this code in the worst case.
 *
 * \warning Intialisation for punctured binary 2-corrector BCH code for the worst
 *          case ONLY.
 *          The initialisation parameters were finding by experimentation on small
 *          codes.
 *
 * \param[out] C the code to initialize
 * \param[in] parameter of the punctured binary 2-corrector BCH code.
 *            it can take for values 4, 5, 6, 7, 8, 9, 10 and 11
 */
void init_PBCH2_code(Code *C, const uint parameter);
/**
 * \fn void init_PBCH2_codes(Code *C, const uint start, const uint nb);
 *
 * \brief initialize the codes array C of size nb with the parameters of punctured 
 *        binary 2-corrector BCH codes of smallest parameter is 'parameter'. 
 *        This initialization permits ONLY to use these codes in the worst case.
 *
 * \warning Intialisation for punctured binary 2-corrector BCH codes for the worst
 *          case ONLY.
 *          The initialisation parameters were finding by experimentation on small
 *          codes.
 *
 * \param[out] C the codes array to initialize
 * \param[in] start the smallest parameter of punctured binary 2-corrector BCH codes
 * \param[in] nb the number of codes in the array
 */
void init_PBCH2_codes(Code *C, const uint start, const uint nb);
/**
 * \fn void PBCH2_codes_WC(const char* filename);
 *
 * \brief Create or erase the file filename, and print inside the worst cases
 *        of stegoschemes based on punctured binary 2-corrector BCH codes.
 *
 * \warning The initialisation parameters were finding by experimentation on small
 *          codes.
 *
 * \param[in] filename the name of the output file
 */
void PBCH2_codes_WC(const char* filename);
/**
 * \fn void PBCH2_codes_AC(const char* filename);
 *
 * \brief Create or erase the file filename, and print inside the average cases
 *        of stegoschemes based on punctured binary 2-corrector BCH codes.
 *
 * \warning The initialisation parameters were finding by experimentation on small
 *          codes.
 *
 * \param[in] filename the name of the output file
 */
void PBCH2_codes_AC(const char* filename);

/**
 * \fn void init_BCH3_code(Code *C, const uint parameter);
 *
 * \brief Initialize the code C as a binary 3-corrector BCH
 *        code with parameter.
 *
 * \warning The initialization is for the binary 3-corrector BCH codes
 *          for the worst case.
 *
 * \param[out] C code to initialize
 * \param[in] parameter to define the binary 3-corrector BCH code.
 *            It can take as for value 4, 5, 6, 7, 8 and 9
 */
void init_BCH3_code(Code *C, const uint parameter);
/**
 * \fn void init_BCH3_codes(Code *C, const uint start, const uint nb);
 *
 * \brief initialize the codes array C of size nb with the parameters of 
 *        binary 3-corrector BCH codes of smallest parameter is 'parameter'. 
 *        This initialization permits ONLY to use these codes in the worst case.
 *
 * \warning Intialisation for binary 3-corrector BCH codes for the worst
 *          case ONLY.
 *          The initialisation parameters were finding by experimentation on small
 *          codes.
 *
 * \param[out] C the codes array to initialize
 * \param[in] start the smallest parameter of binary 3-corrector BCH codes
 * \param[in] nb the number of codes in the array
 */
void init_BCH3_codes(Code *C, const uint start, const uint nb);
/**
 * \fn void BCH3_codes_WC(const char* filename);
 *
 * \brief Create or erase the file filename with the data of the
 *        binary 3-corrector BCH codes for the worst case.
 *
 * \param[in] filename the name of the output file
 */
void BCH3_codes_WC(const char* filename);
/**
 * \fn void BCH3_codes_AC(const char* filename);
 *
 * \brief Create or erase the file filename with the data of the
 *        binary 3-corrector BCH codes for the average case.
 *
 * \param[in] filename the name of the output file
 */
void BCH3_codes_AC(const char* filename);


/**
 * \fn void init_PBCH3_code(Code *C, const uint parameter);
 *
 * \brief Initialize the code C as a punctured binary 3-corrector BCH
 *        code with parameter.
 *
 * \warning The initialization is for the punctured binary 3-corrector BCH codes
 *          for the worst case.
 *
 * \param[out] C code to initialize
 * \param[in] parameter to define the punctured binary 3-corrector BCH code.
 *            It can take as for value 4, 5, 6, 7, 8 and 9
 */
void init_PBCH3_code(Code *C, const uint parameter);
/**
 * \fn void init_PBCH3_codes(Code *C, const uint start, const uint nb);
 *
 * \brief initialize the codes array C of size nb with the parameters of punctured
 *        binary 3-corrector BCH codes of smallest parameter is 'parameter'. 
 *        This initialization permits ONLY to use these codes in the worst case.
 *
 * \warning Intialisation for punctured binary 3-corrector BCH codes for the worst
 *          case ONLY.
 *          The initialisation parameters were finding by experimentation on small
 *          codes.
 *
 * \param[out] C the codes array to initialize
 * \param[in] start the smallest parameter of punctured binary 3-corrector BCH codes
 * \param[in] nb the number of codes in the array
 */
void init_PBCH3_codes(Code *C, const uint start, const uint nb);
/**
 * \fn void PBCH3_codes_WC(const char* filename);
 *
 * \brief Create or erase the file filename with the data of the punctured
 *        binary 3-corrector BCH codes for the worst case.
 *
 * \param[in] filename the name of the output file
 */
void PBCH3_codes_WC(const char* filename);
/**
 * \fn void PBCH3_codes_AC(const char* filename);
 *
 * \brief Create or erase the file filename with the data of the punctured
 *        binary 3-corrector BCH codes for the average case.
 *
 * \param[in] filename the name of the output file
 */
void PBCH3_codes_AC(const char* filename);


/**
 * \fn void init_BCH4_code(Code *C, const uint parameter);
 *
 * \brief Initialize the code C as a binary 4-corrector BCH
 *        code with parameter.
 *
 * \param[out] C code to initialize
 * \param[in] parameter to define the binary 4-corrector BCH code.
 *            It can take as for value 4, 5 and 6
 */
void init_BCH4_code(Code *C, const uint parameter);
/**
 * \fn void init_BCH4_codes(Code *C, const uint start, const uint nb);
 *
 * \brief initialize the codes array C of size nb with the parameters of 
 *        binary 4-corrector BCH codes of smallest parameter is 'parameter'. 
 *
 * \warning The initialisation parameters were finding by experimentation
 *          on small codes.
 *
 * \param[out] C the codes array to initialize
 * \param[in] start the smallest parameter of binary 4-corrector BCH codes
 * \param[in] nb the number of codes in the array
 */
void init_BCH4_codes(Code *C, const uint start, const uint nb);
/**
 * \fn void BCH4_codes_WC(const char* filename);
 *
 * \brief Create or erase the file filename with the data of the
 *        binary 4-corrector BCH codes for the worst case.
 *
 * \param[in] filename the name of the output file
 */
void BCH4_codes_WC(const char* filename);
/**
 * \fn void BCH4_codes_AC(const char* filename);
 *
 * \brief Create or erase the file filename with the data of the
 *        binary 4-corrector BCH codes for the average case.
 *
 * \param[in] filename the name of the output file
 */
void BCH4_codes_AC(const char* filename);


#endif /* __BCH_H__ */
