/**
 * \file code.h
 *
 * \brief Mini librairy to manipulate error correcting
 * codes paramaeters, which are important for 
 * code based steganography.
 *
 * \author Morgan Barbier <morgan.barbier@ensicaen.fr>
 *
 *
 * \version 1.0
 *
 * \date 2014-10-15
 *
 * \copyright This file is part of stegocurve.

 stegocurve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 stegocurve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with stegocurve. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __CODE_H__
#define __CODE_H__

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "tools/definition.h"

/**
 * \struct CODE code.h
 *
 * \brief Structure where is managed the codes parameters.
 */
struct CODE
{
  /** Length of the code [uint] */
  uint n;
  /** Dimension of the code [uint] */
  uint k;
  /** Covering radius of the code [uint] */     
  uint cr;
  /** Average covering radius of the code [double] */
  double a_cr;
  /** Embedding efficiency of the stegoscheme [double] */
  double ee;
  /** Average embedding efficiency of the stegoscheme [double] */
  double a_ee;
  /** Relative payload of the stegoscheme [double] */
  double rp;
};
/**
 * \typedef Code
 *
 * equivalent to struct CODE
 */
typedef struct CODE Code;


/**
 * \fn void malloc_codes(Code **C, const uint nb);
 *
 * \brief Allocates C a codes array of size nb.
 *
 * \param[out] C codes array to allocate
 * \param[in] nb the size of the codes array
 */
void malloc_codes(Code **C, const uint nb);
/**
 * \fn void free_codes(Code **C);
 *
 * \brief desallocates C a codes array.
 *
 * \param[out] C codes array to desallocate
 */
void free_codes(Code **C);
/**
 * \fn void init_code_WC(Code *C, const uint n, const uint k, const uint cr);
 *
 * \brief Initialize the code and stegoscheme parameters in the worst case.
 *
 * \param[out] C the code where we initialize the parameters
 * \param[in] n the length of the code
 * \param[in] k the dimension of the code
 * \param[in] cr covering radius of the code
 */
void init_code_WC(Code *C, const uint n, const uint k, const uint cr);
/**
 *\fn void init_code_AC(Code *C, const uint n, const uint k, const double a_cr);
 *
 * \brief Initialize the code and stegoscheme parameters in the average case.
 *
 * \param[out] C the code where we initialize the parameters
 * \param[in] n the length of the code
 * \param[in] k the dimension of the code
 * \param[in] a_cr average covering radius of the code
 */
void init_code_AC(Code *C, const uint n, const uint k, const double a_cr);
/**
 * \fn void update_code_WC(Code *C, const uint cr);
 *
 * \brief Update the code and stegoscheme parameters for the worst case.
 *
 * \param[in,out] C the code where we initialize the parameters
 * \param[in] cr covering radius of the code
 */
void update_code_WC(Code *C, const uint cr);
/**
 * \fn void update_code_AC(Code *C, const double a_cr);
 *
 * \brief Update the code and stegoscheme parameters for the average case.
 *
 * \param[in,out] C the code where we initialize the parameters
 * \param[in] a_cr average covering radius of the code
 */
void update_code_AC(Code *C, const double a_cr);
/**
 * \fn void init_generic_codes(Code *C,
			const uint start,
			const uint nb,
			void (*init_fct)(Code*, const uint));
 *
 * \brief Generic function to initialize an array of codes.
 *
 * \param[out] C the array of codes to initialize
 * \param[in] start the smallest parameter
 * \param[in] nb the size of the array
 * \param[in] init_fct the function pointer to initialize the array
 */
void init_generic_codes(Code *C,
			const uint start,
			const uint nb,
			void (*init_fct)(Code*, const uint));

/**
 * \fn uint get_len(const Code *C);
 *
 * \brief Gives the length of the given code
 *
 * \param[in] C the error correcting code
 * \return length of the code as 'uint'
 */
uint get_len(const Code *C);
/**
 * \fn uint get_dim(const Code *C);
 *
 * \brief Gives the dimension of the given code
 *
 * \param[in] C the error correcting code
 * \return dimension of the code as 'uint'
 */
uint get_dim(const Code *C);
/**
 * \fn uint get_CR(const Code *C);
 *
 * \brief Gives the covering radius of the given code
 *
 * \param[in] C the error correcting code
 * \return covering radius of the code as 'uint'
 */
uint get_CR(const Code *C);
/**
 * \fn uint get_codim(const Code *C);
 *
 * \brief Gives the co-dimension of the given code
 *
 * \param[in] C the error correcting code
 * \return co-dimension of the code as 'uint'
 */
uint get_codim(const Code *C);
/**
 * \fn double get_EE(const Code *C);
 *
 * \brief Gives the embedding efficiency of the 
 *        stegoscheme based on the given code.
 *
 * \param[in] C the error correcting code
 * \return embedding efficiency as 'double'
 */
double get_EE(const Code *C);
/**
 * \fn double get_RP(const Code *C);
 *
 * \brief Gives the relative payload of the 
 *        stegoscheme based on the given code.
 *
 * \param[in] C the error correcting code
 * \return relative payload as 'double'
 */
double get_RP(const Code *C);
/**
 * \fn double get_ACR(const Code *C);
 *
 * \brief Gives the average covering radius of the 
 *        stegoscheme based on the given code.
 *
 * \param[in] C the error correcting code
 * \return average covering raius of the code as 'double'
 */
double get_ACR(const Code *C);
/**
 * \fn double get_AEE(const Code *C);
 *
 * \brief Gives the average embedding efficiency of the 
 *        stegoscheme based on the given code.
 *
 * \param[in] C the error correcting code
 * \return average embedding efficiency of the code as 'double'
 */
double get_AEE(const Code *C);

/**
 * \fn void display_datum_WC(FILE *out, const Code *C);
 *
 * \brief prints the relative payload and the embedding efficiency
 *        of the stegoscheme based on the code 'C' in the streamming
 *        file 'out', which permits to built the datafile to plot
 *        the curve in the worst case.
 *
 * \param[out] out the streaming file to write in
 * \param[in] C the code
 */
void display_datum_WC(FILE *out, const Code *C);
/**
 * \fn void display_datum_AC(FILE *out, const Code *C);
 *
 * \brief prints the relative payload and the average embedding efficiency
 *        of the stegoscheme based on the code 'C' in the streamming
 *        file 'out', which permits to built the datafile to plot
 *        the curve in the average case.
 *
 * \param[out] out the streaming file to write in
 * \param[in] C the code
 */
void display_datum_AC(FILE *out, const Code *C);
/**
 * \fn void display_data_WC(FILE *out, const Code *C, const uint size);
 *
 * \brief prints the relative payloads and the embedding efficiencies
 *        of the stegoschemes based on the codes array 'C' in the streamming
 *        file 'out', which build the datafile to plot
 *        the curve in the worst case.
 *
 * \param[out] out the streaming file to write in
 * \param[in] C the code
 * \param[in] size the number of codes in the array C
 */
void display_data_WC(FILE *out, const Code *C, const uint size);
/**
 * \fn void display_data_AC(FILE *out, const Code *C, const uint size);
 *
 * \brief prints the relative payloads and the average embedding efficiencies
 *        of the stegoschemes based on the codes array 'C' in the streamming
 *        file 'out', which build the datafile to plot
 *        the curve in the average case.
 *
 * \param[out] out the streaming file to write in
 * \param[in] C the code
 * \param[in] size the number of codes in the array C
 */
void display_data_AC(FILE *out, const Code *C, const uint size);

/**
 * \fn void display_code_parameters(FILE *out, const Code *C);
 *
 * \brief prints in the streamming file 'out' all parameter of the code
 *        and the associated stegoscheme.
 *
 * \param[out] out the streaming file to write in
 * \param[in] C the code
 */
void display_code_parameters(FILE *out, const Code *C);


/**
 * \fn void generic_codes_generic_cases(const char* filename,
				 const uint start,
				 const uint nb,
				 void (*init_fct)(Code*, const uint, const uint),
				 void (*display_fct)(FILE*, const Code*, const uint));

 *
 * \brief Generic function which initialize any familly of codes and display the result
 *        with any appropriate functions.
 *
 * \param[in] filename the name of the output file
 * \param[in] start the smallest parameter of the familly
 * \param[in] nb the number of codes in the familly
 * \param[in] init_fct pointer on initialization function 
 * \param[in] display_fct pointer on display function
 */
void generic_codes_generic_cases(const char* filename,
				 const uint start,
				 const uint nb,
				 void (*init_fct)(Code*, const uint, const uint),
				 void (*display_fct)(FILE*, const Code*, const uint));


#endif /* __CODE_H__ */
