/**
 * \file entropy.h
 *
 * \brief Mini librairy to manipulate entropy
 *
 * \author Morgan Barbier <morgan.barbier@ensicaen.fr>
 *
 * \version 1.0
 *
 * \date 2014-10-15
 *
 * \copyright This file is part of stegocurve.

 stegocurve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 stegocurve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with stegocurve. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __ENTROPY_H__
#define __ENTROPY_H__

#include <stdio.h>
#include <math.h>
#include <assert.h>
#include "codes/code.h"

/**
 * \fn double q_log(const double x, const uint q);
 *
 * \brief returns the 'q'-ary logarithm of the variable 'x'.
 *
 * \param[in] x the variable to compute its 'q'-ary logarithm
 * \param[in] q the basis of the logarithm
 * \returns 'q'-ary logarithm of 'x'
 */
double q_log(const double x, const uint q);
/**
 * \fn double entropy(const double x, const uint q);
 * 
 * \brief compute the 'q'-ary entropy of the variable 'x'.
 *
 * \param[in] x the variable to compute its 'q'-ary entropy
 * \param[in] q the basis of the entropy
 * \returns 'q'-ary entropy of the variable 'x'
 */
double entropy(const double x, const uint q);
/**
 * \fn void entropy_print_theoretical_bound(FILE *out, const double step, const uint q);
 *
 * \brief print into streamming file 'out', the inverse function of 'q'-ary 
 *        entropy for different value from 'step' to (q-1)/q by step.
 *
 * \param[in] out the streamming file to write in
 * \param[in] step is the smallest difference between two values
 * \param[in] q is the basis of the entropy
 */
void entropy_print_theoretical_bound(FILE *out, const double step, const uint q);
/**
 * \fn void theoretical_bound(const char *filename, const double step, const uint q);
 *
 * \brief Create or erase the file filename with the data to plot the theoretical 
 *        bound for stegoscheme based on error correcting code.
 *
 * \param[in] filename of the output file
 * \param[in] step precision between two closest values
 * \param[in] q size of alphabet. (e.g. 2 for binary)
 */
void theoretical_bound(const char *filename, const double step, const uint q);
#endif /* __ENTROPY_H__ */
