/**
 * \file stegocurve_config.h
 *
 * \brief Header file which provides a structure to manage
 * the behavior of stegocurve.
 *
 * \author Morgan Barbier <morgan.barbier@ensicaen.fr>
 *
 * \version 1.2
 *
 * \date 2023-03-21
 *
 * \copyright This file is part of stegocurve.

 stegocurve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 stegocurve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with stegocurve. If not, see <http://www.gnu.org/licenses/>.

 */
#ifndef __STEGOCURVE_CONFIG_H__
#define __STEGOCURVE_CONFIG_H__

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "tools/definition.h"
#include "tools/stegocurve_args.h"

#include "codes/codes.h"
#include "tools/entropy.h"

#include "plotters/plotters.h"


/* For directory creation */
#ifdef __linux__
#include <sys/stat.h>
#endif /* __linux__ */
#ifdef __WIN32__
#include <dir.h>
#endif /* __WIN32__ */


#define LINE_LENGTH 200 /**< \def #define LINE_LENGTH 200 Maximum length for line */

/**
 * \enum action_type to determine the type of the action to treat
 */
typedef enum
  {
    generate,   /**< for the data generation step */
    gnuplot,    /**< to plot with gnuplot */
    matplotlib, /**< to plot with matplotlib */
    tikz,       /**< to plot with tikz pure */
    pgfplots,   /**< to plot with tikz and pgfplots */
    error       /**< to manage an error */
  } action_type; 

/**
 * \struct STEGOCURVE_ACTION
 *
 * \brief structure to manage the behavior of the
 * stegocurve from command line or config file.
 */
struct STEGOCURVE_ACTION
{
  stegocurve_args *data;       /**< make the association between the data file and the curve title [stegocurve_args] */
  action_type      action;     /**< action to do [action_type] */
  char            *action_arg; /**<  an extra argument for the action [char*] */
};
/** 
 * \typedef typedef struct STEGOCURVE_ACTION stegocurve_action;
 *
 * \brief equivalent to struct STEGOCURVE_ACTION
 */
typedef struct STEGOCURVE_ACTION stegocurve_action;

/**
 * \fn void generate_all(const char *data_directory);
 *
 * \brief Generate all data define in stegocurve in the 
 *        directory data_directory
 *
 * \param[in] data_directory the name of the directory where
 *            data will write
 */
void generate_all(const char *data_directory);


/**
 * \fn void stegocurve_action_init(stegocurve_action *act);
 *
 * \brief allocate the needed memory for an action
 *
 * \param[in] act gives the pointer to a stegocurve_action
 *            to allocate.
 */
void stegocurve_action_init(stegocurve_action *act);
/**
 * \fn void stegocurve_action_free(stegocurve_action *act);
 *
 * \brief desallocate the memory takes by the
 *        stegocurve_action
 *
 * \param[in] act to desallocate
 */
void stegocurve_action_free(stegocurve_action *act);

/**
 * \fn void stegocurve_action_do(const stegocurve_action *act);
 *
 * \brief from the structure stegocurve_action make the action
 *
 * \param[in] act the action to do
 */
void stegocurve_action_do(const stegocurve_action *act);

/**
 * \fn void extract_value_from_line(char **val, const char *line);
 *
 * \brief allocate and put the value from line into val. This value
 *        must be between ' or ".
 *
 * \param[in] val variable receving the result
 * \param[in] line variable containing all the line
 *
 * \warning This function allocate the needed memory for val
 */
void extract_value_from_line(char **val, const char *line);
/**
 * \fn void stegocurve_read_config_file(stegocurve_action *act, const char * config_filename);
 *
 * \brief The file config_filename will open and read in order to
 *        initialize the act. This is a kind a special parser for
 *        a stegocurve config file.
 *
 * \param[in] act is the stegocurve_action to initialize from config 
 *            file
 * \param[in] config_filename the path and name to the setting file
 *            of stegocurve
 */
void stegocurve_read_config_file(stegocurve_action *act, const char * config_filename);



#endif /* __STEGOCURVE_CONFIG_H__ */
